package com.sheelapps.swtml.examples.texteditor;

import java.util.Vector;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ExtendedModifyEvent;
import org.eclipse.swt.custom.ExtendedModifyListener;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.ToolItem;

import com.sheelapps.swtml.SWTObjectMap;
import com.sheelapps.swtml.SWTParser;

public class TextEditor {
	public static final String START_STYLES_MARK = "[[~Styles~]]";

	static Vector cachedStyles = new Vector();

	public static String newline = System.getProperty("line.separator");

	private static StyledText styledText = null;

	static Color RED = null;

	static Color BLUE = null;

	static Color GREEN = null;

	static String fileName = null;

	static Shell shell;

	public static void main(String[] args) {
		TextEditor ed = new TextEditor();
		ed.open();
	}

	private void open() {
		Display display = Display.getDefault();

		try {
			initializeColors();
			SWTParser parser = new SWTParser();
			shell = (Shell) parser
					.doParse(TextEditor.class
							.getResourceAsStream("/com/sheelapps/swtml/examples/texteditor/texteditor.xml"));

			Rectangle parentRect = display.getPrimaryMonitor().getBounds();
			Point size = shell.getSize();
			shell.setLocation(parentRect.x + (parentRect.width - size.x) / 2,
					parentRect.y + (parentRect.height - size.y) / 2);
			shell.open();
			shell.setSize(800, 600);
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch())
					display.sleep();
			}
			display.dispose();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public static StyledText getStyledText() {
		if (styledText == null) {
			styledText = (StyledText) SWTObjectMap.getInstance().getObjectMap()
					.get("styledText");
			styledText.addExtendedModifyListener(new ExtendedModifyListener() {
				public void modifyText(ExtendedModifyEvent e) {
					handleExtendedModify(e);
				}
			});
		}
		return styledText;

	}

	private void initializeColors() {
		Display display = Display.getDefault();
		RED = new Color(display, new RGB(255, 0, 0));
		BLUE = new Color(display, new RGB(0, 0, 255));
		GREEN = new Color(display, new RGB(0, 255, 0));
	}

	private static void handleExtendedModify(ExtendedModifyEvent event) {

		if (event.length == 0)
			return;
		StyleRange style;
		if (event.length == 1
				|| styledText.getTextRange(event.start, event.length).equals(
						styledText.getLineDelimiter())) {
			// Have the new text take on the style of the text to its right
			// (during
			// typing) if no style information is active.
			int caretOffset = styledText.getCaretOffset();
			style = null;
			if (caretOffset < styledText.getCharCount())
				style = styledText.getStyleRangeAtOffset(caretOffset);
			if (style != null) {
				style = (StyleRange) style.clone();
				style.start = event.start;
				style.length = event.length;
			} else {
				style = new StyleRange(event.start, event.length, null, null,
						SWT.NORMAL);
			}
			if (getButton("bold").getSelection())
				style.fontStyle |= SWT.BOLD;
			if (getButton("italic").getSelection())
				style.fontStyle |= SWT.ITALIC;
			style.underline = getButton("underline").getSelection();
			style.strikeout = getButton("strikeout").getSelection();
			if (!style.isUnstyled())
				styledText.setStyleRange(style);
		} else {
			// paste occurring, have text take on the styles it had when it was
			// cut/copied
			for (int i = 0; i < cachedStyles.size(); i++) {
				style = (StyleRange) cachedStyles.elementAt(i);
				StyleRange newStyle = (StyleRange) style.clone();
				newStyle.start = style.start + event.start;
				styledText.setStyleRange(newStyle);
			}
		}

	}

	static ToolItem getButton(String id) {
		return (ToolItem) SWTObjectMap.getInstance().getObjectMap().get(id);
	}

	public static void dispose() {
		RED.dispose();
		GREEN.dispose();
		BLUE.dispose();
		shell.dispose();

	}
}
